#ifndef        HMODEL_H
#define HMODEL_H

#include "common.h"

//////////////////////////////////////////////////////////////////////////////////
//	!Please, if you use this code in your own project, give credit to the
//	author (Cameron Sneed - http:\\www.thefloatingpoint.org ). 
//
//	Thanks!
//	
//	Directly below is high precision Pi calculations. Thanks goes to 
//	Chao7 for his help with this!
//
//////////////////////////////////////////////////////////////////////////////////



uniform samplerCUBE         env_s0 ;                
uniform samplerCUBE         env_s1 ;

uniform half4                         env_color        ;        // color.w  = lerp factor
uniform half3x4                        m_v2w                ;

void        hmodel                 (out half3 hdiffuse, out half3 hspecular, half m, half h, half s, float3 point, half3 normal)
{
	///////////////////////////////////////////////////////////////////////////////////////
        // !hscale - Hemisphere lighting
	///////////////////////////////////////////////////////////////////////////////////////
	 half3         nw                         = mul                 (m_v2w,normal);    
        half         hscale                 = h;                                //. *        (.5h + .5h*nw.y);


	///////////////////////////////////////////////////////////////////////////////////////
        // !New Cook-Torrance reflectance model below.
	// 
	///////////////////////////////////////////////////////////////////////////////////////

	half3 N		= normal;						 
  	half3 V 	= normalize		(point.xyz);
	half3 Vx        = mul			(m_v2w,V);			
  	half4 L 	= -Ldynamic_dir;					
	
	///////////////////////////////////////////////////////////////////////////////////////
	//	 The code below is copyright of Cameron Sneed. Algorithms used are copyright
	//	of thier respective owners.
	///////////////////////////////////////////////////////////////////////////////////////
	
	half3	Ldir			=  reflect (-L,N);
	half	Sspec			=  dot(-Ldir,V);
	half4	F_spec			=  tex3D (s_material, half3(hscale, (Sspec/2), m));
	half4	L_sun			=  hscale*2;	
	
	half3	Ha	= normalize(L+V);
	float4 LV = ((saturate(dot(N,Ha))),(saturate(dot(V,Ha))),		
		    (saturate(dot(N,V))),(saturate(dot(L,N))));

 const	float	RI	= 0.01f;
 const	float	Pi	= 3.1415926535;	

	half	ms	= xmaterial;
	float	NH2	= LV.x * LV.x;
	float	m2	= ms * ms;
	float3	vD	= ((RI + (1 - RI) * pow((1 - LV.z), 5.f)),
			  (1 / m2 * NH2 * NH2) * (exp(-((1 - NH2) / (m2 * NH2)))),
			  min(1.f, min((2 * LV.x * LV.w) / LV.y, (2 * LV.x * LV.z) / LV.y)));

	float	Dw	= (1.0f);
	float	Si	= (vD.x * vD.y * vD.z) / (Pi * LV.w * LV.z);
	float	ret	= 0.8f * (hscale) + (LV.w * saturate(Dw * ((0.5f * LV.w) + (0.2f * Si)))) * -L.w;	

		half	light	= (ret);



	///////////////////////////////////////////////////////////////////////////////////////
	//	 The code above is copyright of Cameron Sneed. Algorithms used are copyright
	//	of thier respective owners.
	///////////////////////////////////////////////////////////////////////////////////////
			

        //////////////////////////////////////////////////////////////////////////////////////
        // diffuse color - !Combined 4 texture lookups into only two for both diff and spec
	//////////////////////////////////////////////////////////////////////////////////////
      	half3         e0d               = texCUBE         (env_s0,nw);
        half3         e1d               = texCUBE         (env_s1,nw);
	

	half3         env_d             = env_color.xyz*((lerp(e0d, e1d, env_color.w)));


	
				 env_d*=env_d;
		
	//////////////////////////////////////////////////////////////////////////////////////
	// !New ambient occlusion
	//////////////////////////////////////////////////////////////////////////////////////


		
	




	
	//////////////////////////////////////////////////////////////////////////////////////
	// !Combine to get Diffuse and Specular color and intensity
	//////////////////////////////////////////////////////////////////////////////////////
				

        hdiffuse                        =  (env_d) * (light) + L_ambient.rgb;
        				  					
		                                                           
        hspecular                       =  L_sun *  (saturate(F_spec.w))*(s) ;		// (env_d) * (light.w) * (s);                 
}
	//////////////////////////////////////////////////////////////////////////////////////
	//  !The code below is for Object dynamic and isn't used for our purposes
	//   but I have left it for compatibility. It is not compiled at run-time.
	//////////////////////////////////////////////////////////////////////////////////////


void         hmodel_table        (out half3 hdiffuse, out half3 hspecular, half m, half h, half s, half3 point, half3 normal)
{
        // hscale - something like diffuse reflection
        half         hscale         = h;

        // reflection vector
        half3         v2point        = normalize        (point);
        half3        vreflect= reflect         (v2point,normal);
        half         hspec         = .5h+.5h*dot        (vreflect,v2point);

        // material
          half4         light        = tex3D                (s_material, half3(hscale, hspec, m) );                // sample material

        // diffuse color
        half3         env_d         = texCUBE         (env_s0,normal);

        // specular color
        half3         env_s          = texCUBE         (env_s0,vreflect);

        //
        hdiffuse        = env_d       *light.xyz          + L_ambient.rgb        ;
        hspecular        = env_s        *light.w         * s                ;
}

#endif
