#ifndef	LMODEL_H
#define LMODEL_H
#include "common.h"

////////////////////////////////////////////////////////////////////////////////////////
//	Created by:
//	 (Cameron Sneed - http:\\www.thefloatingpoint.org ). 
//
//	
//	
//	Directly below is high precision Pi calculations. Thanks goes to 
//	Chao7 for his help with this!
//
////////////////////////////////////////////////////////////////////////////////////////



const float cPi = 3.14159265; // 3.14159274
const float4 ePi = float4(
        3.14159e-00,
        2.65358e-06,
        9.79323e-12,
        8.46264e-18
);
 
const float getPi_dp24() {
    float rPi;
    rPi = ePi[3] +
          ePi[2] +
          ePi[1] +
          ePi[0] ;
    return rPi;
}
 
const float getPi_dp18() {
    float rPi;
    rPi = ePi[2] +
          ePi[1] +
          ePi[0] ;
    return rPi;
}
 
const float getPi_dp12() {
    volatile float rPi;
    rPi = ePi[1] +
          ePi[0] ;
    return rPi;
}
 
float4 Pi_p( float4 unk1 : COLOR1, float4 unk2 : COLOR2 ) : COLOR0 {
    // arguments to main are 'uncertain', and so are not const-expression optimized away
    const float3 v1 = float3( 1, 1, 1 );
    float
        NL = unk1[0], NV = unk1[1],
        F = unk1[2],
        Df = unk1[3],
        G = unk2[0],
        Pi = getPi_dp24();
    float Si_1 = (F * Df * G);
    float Si_2 = (Pi * NL * NV);
    return float4( Si_1, Si_2, Si_1 / Si_2, 1 );
}


//////////////////////////////////////////////////////////////////////////////////////////
// Lighting formulas - !Infinite lights (sun). 
// !Set up the vectors for use with Cook-Torrance.
//                                         
//////////////////////////////////////////////////////////////////////////////////////////

uniform samplerCUBE         env_s4 ;

half4 plight_infinity	(half m, half3 point, half3 normal, half4 light_direction)       {				
  	half3 N		= normalize(normal);					 
  	half3 V 	= -normalize		(point);		
  	half3 L 	= -light_direction.xyz;				



///////////////////////////////////////////////////////////////////////////////////////////
// !New Cook-Torrance model. Removed texture lookup to save on memory bandwidth.
// RI = Fresnel index   Pi = return value from assembly at the top of the file.
// 
///////////////////////////////////////////////////////////////////////////////////////////	
//
//	 The below code is copyright of Cameron Sneed. Algorithms used are copyright of
//	thier respective owners.
///////////////////////////////////////////////////////////////////////////////////////////

	
	half3	H	= normalize(L+V);
	
	
	float4  LV 	= ((saturate(dot(N,H))),(saturate(dot(V,H))),		
		    	  (saturate(dot(N,V))),(saturate(dot(L,N))));


 const	float	RI	= 0.01f;
 const	float	Pi	= getPi_dp24();			// (high precision Pi);	

	float  	Ds	= (28/(23 * Pi)) * (1 - pow(1 - (LV.z / 2), 5.0f)) * (1 - pow(1 - (LV.w / 2), 5.0f));
	float	m2	= m * m;
	float	NH2	= (LV.x * LV.x);
	float3	vD	= ((RI + (1 - RI) * pow((1 - LV.z), 5.f)),
			  (1 / m2 * NH2 * NH2) * (exp(-((1 - NH2) / (m2 * NH2)))),
			  min(1.f, min((2 * LV.x * LV.w) / LV.y, (2 * LV.x * LV.z) / LV.y)));

	float2	Si	= ((vD.x * vD.y * vD.z * Ds) / (Pi * LV.w * LV.z),1.0f);
	float	ret	= 1.25f * (LV.w*LV.x) + (LV.w * saturate(Si.y * ((0.5f * LV.w) + (0.2f * Si)))) * -light_direction.w;
	
	return (ret);

}

//////////////////////////////////////////////////////////////////////////////////////////
//  !Below is model lights (fire barrels, lights, et cetera.)
//   Cook-Torrance replaced phong-blinn. Code has been optimized and performance has
//   improved over stock game code.
//
//////////////////////////////////////////////////////////////////////////////////////////
uniform samplerCUBE         env_s3 ;

half4 	plight_local		(half m, half3 point, half3 normal, half4 light_position, half light_range_rsq, out float rsqr)  {
  half3 N		=normalize (normal);							// normal 
  half3 L2P 	= point-light_position.xyz;                         		// light2point 
  half3 V 		= -normalize	(point.xyz);					// vector2eye
  half3 L 		= -normalize	((half3)L2P);					// vector2light

 const	float	RI	= 0.01f;
 const	float	Pi	= getPi_dp24();			// (high precision Pi);
	
	half3	H	= normalize(L+V);

	float4 LV = ((saturate(dot(N,H))),(saturate(dot(V,H))),		
		    (saturate(dot(N,V))),(saturate(dot(L,N))));

	


	float	NH2	= LV.x * LV.x;
	float	m2	= m * m;
	float3	vD	= ((RI + (1 - RI) * pow((1 - LV.z), 5.f)),
			  (1 / m2 * NH2 * NH2) * (exp(-((1 - NH2) / (m2 * NH2)))),
			  min(1.f, min((2 * LV.x * LV.w) / LV.y, (2 * LV.x * LV.z) / LV.y)));

	float2	Si	= ((vD.x * vD.y * vD.z) / (Pi * LV.w * LV.z),1.0f);
	float	ret	= 1.25f * (LV.w*LV.x) + (LV.w * saturate(Si.y * ((0.5f * LV.w) + (0.2f * Si)))) * -light_position.w;
	

  	

	////////////////////////////////////////////////////////////////////////////////////////////////////////
	//  Code above is copyright of Cameron Sneed. Algorithms used are copyright of thier respective owners.
	////////////////////////////////////////////////////////////////////////////////////////////////////////

		rsqr	= dot		(L2P,L2P);					// distance 2 light (squared)
  half  att 	= saturate	(1 - rsqr*light_range_rsq);			// q-linear attenuate

  	 return (att)*(ret);



}

 

 half4	blendp	(half4	value, float4 	tcp)    		{
	

	#ifndef FP16_BLEND  
		value 	+= (half4)tex2Dproj 	(s_accumulator, tcp); 	// emulate blend
	#endif
	return 	value	;
 }
 half4	blend	(half4	value, float2 	tc)			{
	#ifndef FP16_BLEND  
		value 	+= (half4)tex2D 	(s_accumulator, tc); 	// emulate blend
	#endif
	return 	value	;
 }

 #endif
