float4x4 matViewInverse;
float4x4 matWorldViewProjection;
float4x4 matView;
float fBaseTextureRepeat;
float fHeightMapRange;
float4 vLightPosition;
struct VS_INPUT
{
float4 positionWS : POSITION;
float2 texCoord : TEXCOORD0;
float3 vNormalWS : NORMAL;
float3 vBinormalWS : BINORMAL;
float3 vTangentWS : TANGENT;
};
struct VS_OUTPUT
{
float4 position : POSITION;
float2 texCoord : TEXCOORD0;
// Light vector in tangent space, not normalized
float3 vLightTS : TEXCOORD1;
// View vector in tangent space, not normalized
float3 vViewTS : TEXCOORD2;
// Parallax offset vector in tangent space
float2 vParallaxOffsetTS : TEXCOORD3;
// Normal vector in world space
float3 vNormalWS : TEXCOORD4;
// View vector in world space
float3 vViewWS : TEXCOORD5;
};
VS_OUTPUT vs_main( VS_INPUT i )
{
VS_OUTPUT Out = (VS_OUTPUT) 0;
// Transform and output input position
Out.position = mul( matWorldViewProjection, i.positionWS );
// Propagate texture coordinate through:
Out.texCoord = i.texCoord;
// Uncomment this to repeat the texture
// Out.texCoord *= fBaseTextureRepeat;
// Propagate the world vertex normal through:

Out.vNormalWS = i.vNormalWS;
// Compute and output the world view vector:
float3 vViewWS = mul( matViewInverse,
float4(0,0,0,1)) - i.positionWS;
Out.vViewWS = vViewWS;
// Compute denormalized light vector in world space:
float3 vLightWS = vLightPosition - i.positionWS;
// Normalize the light and view vectors and transform
// it to the tangent space:
float3x3 mWorldToTangent =
float3x3( i.vTangentWS, i.vBinormalWS, i.vNormalWS );
// Propagate the view and the light vectors (in tangent space):
Out.vLightTS = mul( mWorldToTangent, vLightWS );
Out.vViewTS = mul( mWorldToTangent, vViewWS );
// Compute the ray direction for intersecting the height field
// profile with current view ray. See the above paper for derivation
// of this computation.
// Compute initial parallax displacement direction:
float2 vParallaxDirection = normalize( Out.vViewTS.xy );
// The length of this vector determines the furthest amount
// of displacement:
float fLength = length( Out.vViewTS );
float fParallaxLength = sqrt( fLength * fLength - Out.vViewTS.z
* Out.vViewTS.z ) / Out.vViewTS.z;
// Compute the actual reverse parallax displacement vector:
Out.vParallaxOffsetTS = vParallaxDirection * fParallaxLength;
// Need to scale the amount of displacement to account for
// different height ranges in height maps. This is controlled by
// an artist-editable parameter:
Out.vParallaxOffsetTS *= fHeightMapRange;
return Out;
} // End of VS_OUTPUT vs_main(..)





